<?php

//##############################################################################
// $Id: setup.pl 589 2013-02-11 11:19:11Z alein $
//
// Copyright(c) 2006-2011 ThreatSTOP, Inc. All Rights Reserved
// 
// NOTICE:  All information contained herein is, and remains the property
// of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and
// technical concepts contained herein are proprietary to ThreatSTOP, Inc.
// and its suppliers and may be covered by U.S. and Foreign Patents, patents
// in process, and are protected by trade secret or copyright law.
// Dissemination of this information or reproduction of this material is
// strictly forbidden unless prior written permission is obtained from
// ThreatSTOP, Inc.
//
//
// ThreatSTOP main function module
//###############################################################################

// ############################################################### //
// ################## MODULE DEFENITIONS PART #################### //
// ############################################################### //

$LCK_FH;							      # Lock file hadler
$LOG_FH;							      # Log file hadler

$CONFIGFILE  = "ts_conf.inc";                                         # Conf file to read
$LOGFILE    = "tsoutput.log";                                         # Log file to write
$LOCKFILE    = "ts.lck";                                              # Lock file to use
$DEBUG_LEVEL = 5;                                                     # Define verbose print level 6 - all types
$LOG_LEVEL   = 6;                          			      # Define verbose log level 6 - all types 
$PRINTINIT   = 1;						      # Define where erros will go, STDOUT/ERRMSG
$LOGINIT     = 0;						      # Define if logging is turned off/on
$ERRORMSG;                                                            # Keeps error messages if no STDOUT opt enabled
$START_EPOCH = time();
$END_EPOCH;

#$DNS_LIST = "64.87.26.147 24.249.204.58";
$dns_server      = $DNSLIST;                                          # "64.87.26.147 24.249.204.58";
$DNS_PROTO       = '+tcp';                                            # Protocol to use - TCP
$DNS_PROTO_ALT   = '+notcp';                                          # Protocol to use - UDP
$DNS_PORT_BASE   = 53;                                                # Port to use (default 53)
$DNS_PORT_ALT    = 5353;                                              # Port to use (alternate 5353)
$DNS_DOMAIN_BASE = 'dns-servers.threatstop.local';                    # Server name to resolve 
$DNS_DOMAIN_ALT  = 'dns-servers.threatstop.com';                      # Alternative server name to resolve
$DNS_SERVERS_ALT = "8.8.8.8 8.8.4.4 208.67.220.220 208.67.222.222";   # Alternate servers ip's to quiery
$DNS_SERVERS_LCL = '127.0.0.1';                                       # Local DNS serves will be stored here
$DNS_MAX_TRIES   = 3;                                                 # Number of tries to update DNS list
$DNS_LIST_NEW;
$DNS_PORT        = 53;    

pcntl_signal(SIGINT, "intHandler");
pcntl_signal(SIGTERM,"intHandler");
pcntl_signal(SIGHUP, "intHandler");
pcntl_signal(SIGQUIT,"intHandler");
pcntl_signal(SIGABRT,"intHandler");

register_shutdown_function('shutdown');

###############################################################
################# FUNCTIONS DEFENITIONS PART ##################
###############################################################

#############################################################
# shutdown : peforms cleanup on exit                        #
#############################################################
function shutdown() {

	global $LOCKFILE;
	unlink($LOCKFILE);
}

#############################################################
# intHandler : Catches different terminate signals and      #
# redirects cleanup to shutdown function                    #
#############################################################
function intHandler($sig) {
	exit(0);
}

#############################################################
#logInit : initialize logging mechanism                     #
#############################################################
function logInit() {

	global $LOGFILE;
	global $LOGINIT;
	global $LOG_FH;
	global $DEBUG_LEVEL;

	if ( $DEBUG_LEVEL > 3 )
		echo "[INFO ] # Initialising logging\n";

	$LOG_FH = @ fopen($LOGFILE,"w");
	if ( ! $LOG_FH ) {
		echo "[WARN] : Can not open log file [$LOGFILE] for writing\n";
		return 1;
	}

        $LOGINIT = 1;
	return 0;
}

#############################################################
# logMsg : logs supplied message into log file              #
#############################################################
function logMsg($msg) {

	global $LOG_FH;
	global $LOGINIT;

	#If logging is not initialised get out
	if ( $LOGINIT == 0 )
		return 0;

	if ( $msg == '' )
		return 0;

	#Set time stamp
	$lt = localtime();
	$ts = sprintf("%02d/%02d/%04d %02d:%02d:%02d",$lt[3],$lt[4]+1,$lt[5] + 1900, $lt[2], $lt[1], $lt[0]);

	$msg = "[$ts] # $msg\n";

	fwrite($LOG_FH,$msg);	
	return 0;
}

#############################################################
# printMsg : prints output if printOut variable is set to 1 #
#############################################################
function printMsg($msg_type, $msg_body) {

	global $LOGINIT;
	global $DEBUG_LEVEL;

	$msg_out = '';
	
	switch($msg_type) {
		case 'ERROR': $msg_out = "[ERROR] # $msg_body";
			      break;
		case 'ALERT': if ( $DEBUG_LEVEL >= 1 )
				$msg_out = "[ALERT] # $msg_body";
			      break;
		case 'WARN': if ( $DEBUG_LEVEL >= 2 )
				$msg_out = "[WARN ] # $msg_body";
			      break;
		case 'INFO': if ( $DEBUG_LEVEL >= 3 )
				$msg_out = "[INFO ] # $msg_body";
			      break;
		case 'DISP': if ( $DEBUG_LEVEL >= 4 )
				$msg_out = "$msg_body";
			     break;
		case 'DEBUG': if ( $DEBUG_LEVEL >= 5 )
				$msg_out = "[DEBUG] # $msg_body";
			      break;
		default:
			 $msg_out = "[SYS_ERROR] # Unrecognized message type [$msg_type]";
	}
	
	if ( $LOGINIT == 1 ) 
		logMsg("$msg_out");

	if ( strcmp($msg_out,'') != 0)  {
		if ( $msg_type == "DEBUG" ) {
			echo "\t$msg_out\n";
		} else {
		       	echo "$msg_out\n";	
		}
	}
        
        return 0;
}

#############################################################
# setLock : locks flow and prevents parallel execution      #
#############################################################
function setLock() {

	global $LCK_FH;
	global $LOCKFILE;

	printMsg("INFO","Locking current execution instance");

	#If lock file exists
	if (file_exists("$LOCKFILE")) {

		#Try to read lock file content. We have locking PID stored there
		$LCK_FH = fopen($LOCKFILE,"r");
		if ( ! $LCK_FH ) {
			printMsg("ERROR","The flow seems to be locked but can not verify lock file content");
			return 1;
		}
		
		#Ok get locking PID number from lock file
		$pid = fgets($LCK_FH);
		fclose($LCK_FH);

		printMsg("ALERT","Proccess is already locked by another [$pid] process and lock file [$LOCKFILE]");
		return 1;
	}

	#No lockfile had been detected. Create one and lock flow
	$LCK_FH = fopen($LOCKFILE,"w");

	#If failed to create lock file
	if ( ! $LCK_FH ) {
		printMsg("ERROR","Can not create [$LOCKFILE] lock file");
		return 1;
	}

	#Write pid into lock file
	$pid = getmypid();
	if ( fwrite($LCK_FH,$pid) === FALSE ) {
		printMsg("ERROR","Can not update [$LOCKFILE] lock file");
		return 1;
	}

	#Lock flow based on lock file
	if ( ! flock($LCK_FH,LOCK_EX) ) {
		printMsg("ERROR","Can not lock current execution...");
		return 1;
	}

	printMsg("DEBUG","Lock established");
	return 0;
}

#############################################################
# setUnLock : Releases lock                                 #
#############################################################
function setUnLock() {
	
	global $LCK_FH;
	global $LOCKFILE;

	printMsg("INFO","Unlocking current execution instance");

	if ( ! flock($LCK_FH, LOCK_UN) ) {
		printMsg("WARN","Can not release lock...");
        } else {
		printMsg("DEBUG","Lock released");
	}

	fclose($LCK_FH);

	return 0;
}

#############################################################
# readLocalBoxNameserversLst :                              #
#############################################################
function readLocalBoxNameserversLst(&$servers_lst) {

        $cmd = "cat /etc/resolv.conf | grep nameserver | /usr/bin/awk '{print \$2}' | tr '\n' ' '";
        $rc = shell_exec($cmd);

        printMsg("INFO","Reading local box configuration");
	preg_replace('/\n/','\s',$rc);
        $servers_lst = $rc;

        return 0;
}

#############################################################
# updateDNSConfFile :                                       #
#############################################################
function updateDNSConfFile() {

	global $CONFIGFILE;
	global $dns_server;
	global $DNS_PORT;

	printMsg("DEBUG","Setting [DNSLIST=$dns_server] parameter");
	printMsg("DEBUG","Setting [PORT=$DNS_PORT] parameter");

	$data = file_get_contents($CONFIGFILE);
	$data = preg_replace('/DNSLIST=.*/',"DNSLIST='$dns_server';",$data);
	$data = preg_replace('/PORT=.*/',"PORT=$DNS_PORT",$data);
	file_put_contents($CONFIGFILE,$data);

	return 0;
}
########################################################################################
# getDNSserversList :                                                                  #
#                    $dig             - dig tool                                       #
#                    $dns_to_ask      - DNS server ip to query                         #
#                    $dns_to_resolve  - DNS domain name to resolve                     #
#                    $dns_protocol    - Net protocol to use TCP/UDP                    #
#                    $dns_query_type  - Quiery type ptr/apl/a (default not set)        #
#                    $dns_status      - ref to hash where query results will be stored #
########################################################################################
function getDNSserversList(	$dig, 
			   	$dns_to_ask, 
                           	$dns_to_resolve, 
                           	$dns_protocol, 
                           	$dns_port, 
                           	$dns_query_type, 
                           	&$dns_status
                           ) {
        $rc;
        $cmd;
        $idx;
	$ar_size;
        $q_line  = array();
        $rc_data = array();
	$rc_line = array();

	printMsg("DEBUG","Quering DNS [$dns_to_ask] server");
	
	# Quering DNS server
	$cmd = "$dig $dns_protocol $dns_query_type -p $dns_port \@$dns_to_ask $dns_to_resolve 2>&1";
        $rc = shell_exec($cmd);
        
	printMsg("DEBUG","$cmd");

        #If dig command failed. Server did not respond go to the next one
	if ( strpos($rc,"connection timed out") !==false ) {
		printMsg('WARN',"DNS server [$dns_to_ask] connection is timed out");
                return 0;
        }

	#################################################
        #Parse command output and extract relevant info # 
        #      - response state                         #
        #      - response time                          #
        #      - DNS servers list                       #
        #################################################
	$rc_data = explode("\n",$rc);
	$ar_size = sizeof($rc_data);

	for($idx=0; $idx<$ar_size; $idx++) {

		# Response status
		if ( preg_match("/;; ->>HEADER<<- opcode: QUERY, status:/", $rc_data[$idx])) {

			$rc_line = explode(' ',$rc_data[$idx]);

			#If status is not NOERROR. Do not use this server. This lines is analysed
                        #prior to other data. Simply return is suficient to not set settings for
                        #this server
                        if ( ! preg_match("/NOERROR/", $rc_line[5], $status)) {
                                printMsg('WARN',"DNS server [$dns_to_ask] respond with bad query [$status[0]] status");
                                return 0;
                        }

			#Query status ok, mark server as good to use 
                        $dns_status[$dns_to_ask]['res_status'] = "NOERROR";
                        continue;
		}

		# Response time
                if ( preg_match("/;; Query time:/",$rc_data[$idx])) {
			$rc_line = explode(' ',$rc_data[$idx]);
                        $dns_status[$dns_to_ask]['res_time'] = $rc_line[3];
                        continue;
                }

		# DNS servers list
                if ( preg_match("/;; ANSWER SECTION:/",$rc_data[$idx])) {

			$idx++; # Take next line

                        while ( ! preg_match("/;;/",$rc_data[$idx])) {
				if ( preg_match("/IN\tA/",$rc_data[$idx])) {

                                        #Split response line. Take DNS ip. Also set for this DNS ip
                                        #fake response time and return status. We will update it
                                        #later
					$rc_line = preg_split('/\s+/',$rc_data[$idx]);
                                        $dns_ip = $rc_line[4];
	
                                        if ( ! isset($dns_status['dnslist'][$dns_ip]) ) {
                                                $dns_status['dnslist'][$dns_ip]    = 1;
                                                $dns_status[$dns_ip]['res_time']   = 0;
                                                $dns_status[$dns_ip]['res_status'] = 'UNKNOWN';
                                        }
                                }
                                $idx++;
                        }
                        #Go back one line it some queries it can me the beginning of QUERY
                        #section and we defenetly want to parse it
                        $idx--;
                        continue;
		}
	}
	
	#Now make sure that we do not take with us servers that are not in DNS list. This can happened
        #if we switched to global servers list. We do not want them only resolve through them
	foreach ($dns_status as $ips => $data ) {

                if ( strcmp($ips,'dnslist') == 0 ) {
                        #Scip this key. It holds DNS list
                        continue;
                }

                if ( ! isset($dns_status['dnslist'][$ips]) ) {
                        printMsg("DEBUG","Removing server [$ips] from the list ");
                        unset($dns_status[$ips]);
                }
        }

	return 0;
}

#################################################################################
# setNewDNSserversList : checks DNS servers response time and put the quickest  #
#                        to response server at the begining of the list         # 
#################################################################################
function setNewDNSserversList( $dns_res, &$dns_res_lst ) {

        $dns_tmp;
        $res_time = 0;
        $res_server;
        $cmd;
        $rv;

	printMsg("DEBUG","Determining the quickest to respond DNS server");

        #Look for server with shortest response time
	foreach ($dns_res['dnslist'] as $dns => $state ) {

                $srv_res_time = $dns_res[$dns]['res_time'];

                #If this server failed to respond than there is no response time
                #setting. Set it anyway to 10000 to make sure we do not choose this server
		if ( ! isset($srv_res_time) )
	                $srv_res_time = 10000;

                if ( $res_time == 0 ) {
                        #This is the first item in the list
                        $res_time = $srv_res_time;
                        $res_server = $dns;
                        continue;
                }

                if ( $res_time > $srv_res_time ) {
                        $res_time = $srv_res_time;
                        $res_server = $dns;
                        continue;
                }
        }

        #Update the $dns_list variable with new servers set. Put the quickest server at the 
        #begining of the list
        $dns_tmp = $res_server;

	foreach ($dns_res['dnslist'] as $dns => $state ) {
                if ( strcmp($dns,$res_server) == 0 ) #Scip we already have put this server in the list 
			continue;
                $dns_tmp .= " $dns";
        }

	$dns_qck = strstr($dns_tmp, ' ', true);
	printMsg("DEBUG","Quickest to respond server - $dns_qck");

        $dns_res_lst = $dns_tmp;
	return 0;
}

##################################################################
# compareDNSlists :                                              #
#                   $dns_list - list with DNS ips                #
#                   $dns_res  - hash with results of DNS quering #
##################################################################
function compareDNSlists($dns_list,$dns_res) {

	global $dns_server;

	printMsg("INFO","Comparing DNS lists");

        #For each DNS found in both list increase count. Than see what DNS servers
        #counters had been left intact
	$dns_lst = preg_split('/\s/',$dns_list);
        foreach ( $dns_lst as $dns ) {

                #This entry is in two lists. Increase hash counter  
                if ( isset($dns_res[$dns]) ) {
                        $dns_res['dnslist'][$dns] = 2;
                        continue;
                }

                #This entry does not appear in second list. Means two lists are 
                #different. No need to continue. Use new list for future checks
                if ( ! isset($dns_res[$dns]) ) {
                        printMsg("DEBUG","Old and new lists are differnet");
                        return 1;
                }
        }

        #Now see if there are entries in dns_list that were not updated by previous loop
        #meaning there are entries in new list which are not in original list. If found
        #no need to continue. Use new list for future checks 
	foreach ( $dns_res['dnslist'] as $dns => $dk ) {
                if ( $dns_res['dnslist'][$dns] == 1 ) {
                        printMsg("DEBUG","DNS lists are different for [$dns]");
                        return 1;
                }
        }

        #Both lists are similar
        printMsg("DEBUG","Both lists are similar");
	$dns_server = $dns_list;
	
	return 0;
}

###############################################################################
# updateDNSlistsDo :                                                          #
#                    $dns_lists   - list of DNS servers ip's to run check on  #
#                    $dns_domain  - server full name to resolve               #
#                    $dns_proto   - protocol to use tcp/udp                   #
#                    $dns_res_lst - DNS list created by our quieries          #
#                    $dns_res     - hash with query parsed results            #
###############################################################################
function updateDNSlistsDo($dig_tool,$dns_list, $dns_domain, $dns_proto, $dns_port, &$dns_res_lst) {

	global $DNS_DOMAIN_BASE;
	global $DNS_DOMAIN_ALT;
	global $DNS_PORT_BASE;
	global $DNS_PORT_ALT;
	global $DNS_PROTO_ALT;
	global $DNS_SERVERS_ALT;
	global $DNS_SERVERS_LCL;
	global $dns_server;

	$dns_res = array();
        $dns_cmp;

        printMsg("INFO","Testing [$dns_list] servers");

	#foreach server in the DNS list query it and extract returned list of DNS servers.
        #store it in hash for future comparison with original DNS list
	$ips = preg_split('/\s/',$dns_list);

        foreach ($ips as $dns ) {
                printMsg("DEBUG","getDNSserversList $dig_tool,$dns,$dns_domain,$dns_proto,$dns_port,'',$dns_res");
                if (getDNSserversList($dig_tool,$dns,$dns_domain,$dns_proto,$dns_port,'',$dns_res) == 1 )
			return 1;
        }

	#If no entries were created in $dns_res this means all calls to currently defined DNS servers
        #failed go back and re-rty with differnet options
	$cnt_elm = count($dns_res); 
        if ( $cnt_elm <= 0 ) {

                printMsg("WARN","All servers for domain failed to resolve...trying to fall back...");

                if ( strcmp($dns_domain,$DNS_DOMAIN_BASE) == 0 &&
                     strcmp($dns_port,$DNS_PORT_BASE) == 0 ) {         #we testes for base domain
			printMsg("DEBUG","We have tested for base domain [$dns_domain].. rc=2");
			return 2;
		}

                if ( strcmp($dns_domain,$DNS_DOMAIN_BASE) == 0 &&
                     strcmp($dns_port,$DNS_PORT_ALT) == 0 ) {          #we tried for base domain on alternate port 
			printMsg("DEBUG","We have tested for base domain [$dns_domain] on alternate port [$dns_port] ... rc=3");
			return 3;
		}

                if ( strcmp($dns_domain,$DNS_DOMAIN_ALT) == 0  &&
                     strcmp($dns_list,$dns_server) == 0 ) {            #we testes for alt domain on base port
			printMsg("DEBUG","We have tested for alt domain [$dns_domain] on base port [53] ... rc = 4");
			return 4;
		}

                if ( strcmp($dns_domain,$DNS_DOMAIN_ALT) == 0  &&
                     strcmp($dns_list,$DNS_SERVERS_ALT) == 0 ) {       #we tested for global serves on base port
			printMsg("DEBUG","We have tested for global serves [$dns_domain] on base port... rc=5");
			return 5;
		}

                if ( strcmp($dns_domain,$DNS_DOMAIN_ALT) == 0  &&
                     strcmp($dns_list,$DNS_SERVERS_LCL) == 0   &&
                     strcmp($dns_proto,$DNS_PROTO) == 0 ) {            #we tested for local defined servers on base port
			printMsg("DEBUG","We have tested for local defined servers [$dns_domain] on base port... rc=6");
			return 6;
		}

                if ( strcmp($dns_proto,$DNS_PROTO_ALT) == 0 &&
                     strcmp($dns_list,$DNS_SERVERS_LCL) == 0 ) {       #we tested for UDP protocol on base port
			printMsg("DEBUG","We have tested for UDP protocol on base port... rc=7");
			return 7;
		}

                return 1;                                             #Unrecognized case
        }

	#Find the quickes to reponse DNS server put it at the beggining of the list. 
        if ( setNewDNSserversList($dns_res,$dns_res_lst) != 0 )
		return 1;

        #Ok, we have results of DNS query DNS resulting list. Compare it to list we start
        #quering with. Hopefully they will be completly similar
        if ( compareDNSlists($dns_list,$dns_res) == 0 )
		return 8;

        #The DNS servers respond ok, but returned DNS list is differnet than the original one
        #go back and re-run operation with new list settings
        return 9;
}


####################################################################################
# updateDNSlist :                                                                  #
#                 $DNS_DOMAIN_BASE = dns-servers.threatstop.local                  #
#                 $DNS_DOMAIN_ALT  = dns-servers.threatstop.com                    #
#                 $DNS_SERVERS_ALT = 8.8.8.8 8.8.4.4 208.67.220.220 208.67.222.222 #
#                 $DNS_PROTO       = +tcp                                          #
#                 $DNS_PROTO_ALT   = +notcp                                        #
#                 $DNS_MAX_TRIES   = 3                                             #
####################################################################################
function updateDNSlist($dig_tool,$dns_list,$dns_port) {

	global $DNS_DOMAIN_BASE;
	global $DNS_DOMAIN_ALT;
	global $DNS_PROTO;
	global $DNS_PORT_BASE;
	global $DNS_PORT_ALT;
	global $DNS_MAX_TRIES;
	global $DNS_SERVERS_ALT;
	global $dns_server;

        $dns_dom         = $DNS_DOMAIN_BASE;
        $dns_proto       = $DNS_PROTO;
        $try_num         =  1;
        $dns_local_lst;
        $dns_results_lst = array();
        $rc;

        #If not defined port param use default 53
	if ( ! isset($dns_port) ) 
	        $dns_port = $DNS_PORT_BASE;

        printMsg("INFO","Updating DNS lists");

        #Try to get DNS servers list until or success is reached or all options failed
	while ( 1 ) {
		
		#Do update DNS action. Make decesign based on return value 
		printMsg("DEBUG","updateDNSlistsDo $dig_tool,$dns_list, $dns_dom, $dns_proto, $dns_port, $dns_results_lst");
                $rc = updateDNSlistsDo($dig_tool,$dns_list, $dns_dom, $dns_proto, $dns_port, $dns_results_lst);

                #Something wrong happened... abort ...
                if ( $rc == 1 ) {
                        printMsg("ERROR","DNS resolving operation failed ...");
                        return 1;
                }

                #All DNS servers in current list failed to respond
                #switch to alternate port
                if ( $rc == 2 ) {
                        $dns_port = $DNS_PORT_ALT;
                        printMsg("INFO","Switching to alternate port [$dns_port]");
                        printMsg("DEBUG","dns list  - [$dns_list]");
                        printMsg("DEBUG","dns_dom   - [$dns_dom]");
                        printMsg("DEBUG","dns_proto - [$dns_proto]");
                        printMsg("DEBUG","dns_port  - [$dns_port]");
                        continue;
                }

		#All DNS servers in current list failed to respond
                #switch to alternate domain
                if ( $rc == 3 ) {
                        $dns_dom = $DNS_DOMAIN_ALT;
                        $dns_port = $DNS_PORT_BASE;
                        printMsg("INFO","Switching to alternate domain [$dns_dom]");
                        printMsg("DEBUG","dns list  - [$dns_list]");
                        printMsg("DEBUG","dns_dom   - [$dns_dom]");
                        printMsg("DEBUG","dns_proto - [$dns_proto]");
                        printMsg("DEBUG","dns_port  - [$dns_port]");
                        continue;
                }

                #All DNS servers in current list failied to respond and
                #also alternate domain had been tried. Switch to alt
                #servers list
                if ( $rc == 4 ) {

                        $dns_list = $DNS_SERVERS_ALT;
                        $dns_dom  = $DNS_DOMAIN_ALT;
                        $dns_port = $DNS_PORT_BASE;

                        printMsg("INFO","Switching to alternate global servers list [$dns_list]");
                        printMsg("DEBUG","dns list  - [$dns_list]");
                        printMsg("DEBUG","dns_dom   - [$dns_dom]");
                        printMsg("DEBUG","dns_proto - [$dns_proto]");
                        printMsg("DEBUG","dns_port  - [$dns_port]");
                        continue;
                }

                #OK, all servers from the GLOBAL servers list had been tried 
                #and failed trying to fallback to nameservers list defined on 
                #the box resolve.conf file
                if ( $rc == 5 ) {

                        readLocalBoxNameserversLst($dns_local_lst);
                        $dns_dom         = $DNS_DOMAIN_ALT;
                        $dns_list        = $dns_local_lst;
                        $DNS_SERVERS_LCL = $dns_local_lst;
                        $dns_port        = $DNS_PORT_BASE;

                        printMsg("INFO","Switching localy defined nameservers list");
                        printMsg("DEBUG","dns list  - [$dns_list]");
                        printMsg("DEBUG","dns_dom   - [$dns_dom]");
                        printMsg("DEBUG","dns_proto - [$dns_proto]");
                        printMsg("DEBUG","dns_port  - [$dns_port]");
                        continue;
                }

                #All checks failed. Last thing to do is to fall back to UDP protocol and
                #re-do whole process again.  
                if ( $rc == 6 ) {
                        $dns_list  = $dns_server;
                        $dns_dom   = $DNS_DOMAIN_BASE;
                        $dns_proto = $DNS_PROTO_ALT;
                        $dns_port  = $DNS_PORT_BASE;

                        printMsg("INFO","Switchibg to UDP protocol");
                        printMsg("DEBUG","dns list  - [$dns_list]");
                        printMsg("DEBUG","dns_dom   - [$dns_dom]");
                        printMsg("DEBUG","dns_proto - [$dns_proto]");
                        printMsg("DEBUG","dns_port  - [$dns_port]");
                        continue;
		}

                #We have tried all means and failed ... Abort futher execution ...
                if ( $rc == 7 ) {
                        printMsg("ERROR","Failed to retieve updated DNS servers list");
                        return 1;
                }

                #The servers respond ok but returned DNS list is different
                #than original one. Need to re-run previous procedure with
                #new list settings
                if ( $rc == 9 ) {
                        #Max number of tries reached... Abort futher execution ...
                        if ( $try_num > $DNS_MAX_TRIES ) {
                                printMsg("ERROR","Maximun number of tries to resolve DNS servers list had been reached...aborting");
                                return 1;
                        }

                        #Run again with new list
                        printMsg("DEBUG","Re-running query process again with new DNS list... [${try_num}/${DNS_MAX_TRIES}]");
                        $dns_list   = $dns_results_lst;
                        $try_num++;
                        continue;
                }

                #Everyting ok and lists are similar break the loop
                if ( $rc == 8 ) {
                        printMsg("DEBUG","DNS list check completed");
                        break;
                }
	}

        #Now if DNS list had been changed and we successfully passed verification process update
        #the threatstop.conf file and $dns_server setting with new DNS data
        if ( strcmp($dns_results_lst,$dns_server) != 1 ){
                $dns_server = $dns_results_lst;
        }

        #If we are running on alternate port
        if ( $dns_port != $DNS_PORT_BASE ) {
                $DNS_PORT = $dns_port;
        }

        printMsg("INFO","Updating config file with new DNS servers list");
	if ( updateDNSConfFile() != 0 )
	        return 1;

        return 0;

}
?>
